//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
/*++


Module Name:

    mon.c

Abstract:

    Power Manager Monitor.
    Run in the background to display power state changes in debugger.

Notes:

Revision History:
	ST - adapted from mon.c - platform builder 4.2

--*/
#include "stdafx.h"


#include <windows.h>
#ifdef UNDER_CE
#include <msgqueue.h>
#include <pnp.h>
#include <pm.h>
#endif
#include <dbt.h>


/*  
	Errors related to this on compile may be related to the wrong SDK installation
	Check you are using Embedded Visual C++ 4 and have the appropriate SDKs installed
	see microsoft web page on embedded development for further details
*/

DWORD PowerManagementThread(LPVOID param) ;//count up thread
static HANDLE ghEventExit=NULL,ghPMThread=0;
HANDLE g_evtPM = 0;
static BOOL bRunning = FALSE;
BOOL StartPowerManagement()
{
#ifdef UNDER_CE
	DWORD dwID;
	ghEventExit = CreateEvent(NULL,FALSE,FALSE,NULL);
	g_evtPM = CreateEvent(NULL,TRUE,TRUE,NULL);
	ghPMThread = CreateThread(NULL,0 , PowerManagementThread ,(void*)ghEventExit,0, &dwID);	
	//PowerManagementThread( (void*)ghEventExit);
#endif
	return TRUE;
}
BOOL StopPowerManagement()
{
#ifdef UNDER_CE
	SetEvent(ghEventExit);
	bRunning = FALSE;
	WaitForSingleObject(ghPMThread,2000);
	CloseHandle(ghEventExit); ghEventExit=0;
#endif
	return TRUE;
}
UINT ConvertByteArray(BYTE* array, int offset)
{
	UINT res = 0;
	res += array[offset];
	res += array[offset+1] * (UINT) 0x100;
	res += array[offset+2] * (UINT) 0x10000;
	res += array[offset+3] * (UINT) 0x1000000;
	return res;
}



//****************************************************************************
DWORD PowerManagementThread(LPVOID param)//count up thread
{
#ifdef UNDER_CE 
	// HWND hDlg = (HWND)lpParam;
    DWORD dwSize, dwFlags;
    HANDLE hReg = NULL;
    MSGQUEUEOPTIONS msgopts = {0};
    BYTE pPNPBuf[sizeof(DEVDETAIL) + 200] = {0};
   // DEVDETAIL *pd = (DEVDETAIL*)pPNPBuf;
	GUID guid = {0};//USB2COM_GUID;
    HANDLE pHandles[2] = {0};
	HANDLE hEventExit = (HANDLE) param;
    msgopts.dwSize = sizeof(MSGQUEUEOPTIONS);
    msgopts.dwFlags = 0;
    msgopts.dwMaxMessages = 0;
    msgopts.cbMaxMessage = sizeof(pPNPBuf);
    msgopts.bReadAccess = TRUE;
    bRunning = TRUE;
    pHandles[0] = CreateMsgQueue(NULL, &msgopts);
    pHandles[1] = hEventExit;

    hReg = RequestPowerNotifications(pHandles[0], PBT_RESUME);


    while(bRunning)
    {
        DWORD dwWaitCode = WaitForMultipleObjects( 2, pHandles, FALSE, INFINITE);
        if(WAIT_OBJECT_0 == dwWaitCode)
        {
            if(ReadMsgQueue(pHandles[0], pPNPBuf, sizeof(pPNPBuf), &dwSize, INFINITE, &dwFlags))
            {
				UINT flag = ConvertByteArray(pPNPBuf, 4);
				TCHAR* msg = NULL;
					switch(flag)
					{
						case 65536:
							msg = L"Power On";
							break;
						case 131072:
							msg = L"Power Off";
							break;
						case 262144:
							msg = L"Power Critical";
							break;
						case 524288:
							msg = L"Power Boot";
							break;
						case 1048576:
							msg = L"Power Idle";
							break;
						case 2097152:
							msg = L"Power Suspend";
							
							break;
						case 8388608:
							msg = L"Power Reset";
							break;
						case 0:
							
							ResetEvent(g_evtPM);
							Sleep(6000);
							SetEvent(g_evtPM);
							msg = L"aaaaaaa"; // non power transition messages are ignored
							break;
						default:
							msg = L"Unknown Flag: " + flag;
							break;
					}
					/*int nCount=10;
					ULARGE_INTEGER uFree,uTotal,uFreeTotal;
					
					while(!GetDiskFreeSpaceEx(L"\\Storage Card",&uFree,&uTotal,&uFreeTotal))
					{
						if(!nCount--) break;
						Sleep(200);
					}
					if(nCount<=0) 
						MessageBox(NULL,msg,L"NOSD",MB_OK);
					else MessageBox(NULL,msg,L"SD",MB_OK);
					*/
					// MessageBox(NULL,msg,L"SD",MB_OK);
            }
        }
        else
        {
            // got terminate signal
            break;
        }
    }
    StopPowerNotifications(hReg);
    CloseHandle(pHandles[0]);
#endif
  //  ExitThread(0);
    return 0;
}

